/***************************************************************************
 * Create a map to translate from RNA to amino acids. Translate a 
 * sequence entered by the user. Compile with ../ch6/split.cpp. 
 * ************************************************************************/
#include<fstream>
#include<iostream>
#include<string>
#include<map>
#include "../ch6/split.h"
#include<vector>
#include<cctype>
#include<stdexcept>
using std::cout; using std::cin; using std::endl;
using std::string; using std::map; using std::ifstream;
using std::getline;using std::vector; using std::invalid_argument;

/* Add entries to a map by mapping all subsequent words in 
 * a string to the first word.
 */
void map_codons(map<string,string> & , const vector<string> &);

/* Return a copy of the input string with the spaces removed
 * and characters made uppercase. Throw an 'invalid_argument'
 * if an invalid letter is encountered.
 */ 
string clean_RNA(string &);

/* Return a copy of the input string with the spaces removed
 * and characters made uppercase. 
 */ 
string rough_clean(string &);

/* Return the translation of the string created by replacing
 * each triple of letters in the input string with its value
 * under the map.
 */
string translation(map<string,string>, const string &);

int main()
{
/* Build the map from RNA codons to amino acids from the file codons.txt. 
 * Each line of that file is the abbreviation for an amino acid, followed by 
 * the nucleotide triples that code for it. 
 */ 	
	map<string, string> rna_to_codon;
	ifstream infile("codons.txt");
	if(!infile)
	{
		cout<<"\nUnable to open file.\n";
		exit(1);
	}
	string data_line;
	vector<string> map_info;
	while(getline(infile, data_line))
	{
	
		cout<<data_line<<endl;
		map_info=split(data_line);
		map_codons(rna_to_codon,map_info);
		
	}
/* Translate the RNA string entered by the user to amino acids.
* Output the result.
*/
	cout<<"\nEnter your RNA to get its amino acid string: ";
	string RNA;
	getline(cin,RNA);
	try
	{
		RNA=clean_RNA(RNA);
	}
	catch  (invalid_argument f)
	{
		cout<<endl<<f.what()<<"encountered:"<<
			" Unrecognized codons will be "<<
			"translated as ??? .\n";
		RNA=rough_clean(RNA);
		cout<<"Proceeding with RNA: "<<RNA<<endl;
	}
	
	cout<<endl<<"amino acid sequence: "<<translation(rna_to_codon,RNA)<<endl;	
	return 0;
}

void map_codons(map<string,string> & RNAcodon, const vector<string> & codon_data)
{
	for(vector<string>::const_iterator it=++codon_data.begin(); it!=codon_data.end();
			it++)
		RNAcodon[*it]=*codon_data.begin();
	return;
}

string clean_RNA(string & input_RNA)
{
	string ret="";
	char current;
	for(string::size_type i=0;i<input_RNA.size();i++)
	{
		if(!isspace(input_RNA[i]))
		{
			current=toupper(input_RNA[i]);
			if(current=='A'||current=='C'||current=='G'||current=='U')
				ret+=current;
		
			else
				throw invalid_argument ("invalid character");
		}
	}
	return ret;
}

string rough_clean(string & input_RNA)
{
	string ret="";
	char current;
	for(string::size_type i=0;i<input_RNA.size();i++)
		if(!isspace(input_RNA[i]))
		{
				current=toupper(input_RNA[i]);
				ret+=current;
		}
	return ret;
}
	
string translation(map<string,string> rna_to_codon, const string & rna)
{	
	string ret;
	for(string::size_type i=0;i+2<rna.size();i+=3)
		if(rna_to_codon[rna.substr(i,3)]!="")
			ret+=rna_to_codon[rna.substr(i,3)]+" ";
		else
			ret+="??? ";
			
	return ret;
}
		
		
